﻿using System;
using System.Diagnostics;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace _RealProxy
{
    class Program
    {
        /// <summary>Main</summary>
        static void Main(string[] args)
        {
            // RP1のテスト
            Program.test_RP1();

            // RP2のテスト
            Program.test_RP2();
        }

        /// <summary>RP1のテスト</summary>
        static void test_RP1()
        {
            // 引数に使う

            // 値型
            int i1 = 0;
            int j1 = 0;

            // 参照型
            MyInt i2 = null;
            MyInt j2 = null;

            try
            {
                // 透過プロキシ（RP1）使用手続き
                RP1 rp = new RP1(new TG());
                TG tg = (TG)rp.GetTransparentProxy();

                // メソッドAを実行
                tg.a();

                // メソッドBを実行（値型）
                Debug.WriteLine(string.Format("4＾2 = {0}", tg.b1(4)));

                // メソッドCを実行（参照型）
                Debug.WriteLine(string.Format("5＾2 = {0}", tg.b2(new MyInt(5)).Val));

                // メソッドd1を実行（値型）
                i1 = 6;
                j1 = 5;
                tg.c1(i1, ref j1);
                Debug.WriteLine(string.Format("6 * 5 = {0}", j1.ToString()));

                // メソッドd2を実行（参照型）
                i2 = new MyInt(7);
                j2 = new MyInt(5);
                tg.c2(i2, ref j2);
                Debug.WriteLine(string.Format("7 * 5 = {0}", j2.Val.ToString()));

                // メソッドe1を実行（値型）
                i1 = 8;
                j1 = 0;
                tg.d1(i1, out j1);
                Debug.WriteLine(string.Format("8＾2 = {0}", j1.ToString()));

                // メソッドe2を実行（参照型）
                i2 = new MyInt(9);
                j2 = null;
                tg.d2(i2, out j2);
                Debug.WriteLine(string.Format("9＾2 = {0}", j2.Val.ToString()));
            }
            catch (Exception ex)
            {
                // 例外処理の動作確認
                Debug.WriteLine(ex.ToString());
            }
        }

        /// <summary>RP2のテスト</summary>
        static void test_RP2()
        {
            // 引数に使う

            // 値型
            int i1 = 0;
            int j1 = 0;

            // 参照型
            MyInt i2 = null;
            MyInt j2 = null;

            try
            {
                // 透過プロキシ（RP1）使用手続き
                RP2 rp = new RP2(typeof(TG));
                TG tg = (TG)rp.GetTransparentProxy();

                // メソッドAを実行
                tg.a();

                // メソッドBを実行（値型）
                Debug.WriteLine(string.Format("4＾2 = {0}", tg.b1(4)));

                // メソッドCを実行（参照型）
                Debug.WriteLine(string.Format("5＾2 = {0}", tg.b2(new MyInt(5)).Val));

                // メソッドd1を実行（値型）
                i1 = 6;
                j1 = 5;
                tg.c1(i1, ref j1);
                Debug.WriteLine(string.Format("6 * 5 = {0}", j1.ToString()));

                // メソッドd2を実行（参照型）
                i2 = new MyInt(7);
                j2 = new MyInt(5);
                tg.c2(i2, ref j2);
                Debug.WriteLine(string.Format("7 * 5 = {0}", j2.Val.ToString()));

                // メソッドe1を実行（値型）
                i1 = 8;
                j1 = 0;
                tg.d1(i1, out j1);
                Debug.WriteLine(string.Format("8＾2 = {0}", j1.ToString()));

                // メソッドe2を実行（参照型）
                i2 = new MyInt(9);
                j2 = null;
                tg.d2(i2, out j2);
                Debug.WriteLine(string.Format("9＾2 = {0}", j2.Val.ToString()));
            }
            catch (Exception ex)
            {
                // 例外処理の動作確認
                Debug.WriteLine(ex.ToString());
            }
        }
    }
}
